@@ -1,39 +1,21 @@
 package ExtUtils::ParseXS;
 
-use 5.006;  # We use /??{}/ in regexes
 use Cwd;
 use Config;
 use File::Basename;
 use File::Spec;
-use Symbol;
 
 require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(process_file);
+$VERSION = '2.01';
 
-# use strict;  # One of these days...
+#use strict;  # That'll always be the dream...
 
 my(@XSStack);	# Stack of conditionals and INCLUDEs
 my($XSS_work_idx, $cpp_next_tmp);
 
-use vars qw($VERSION);
-$VERSION = '2.21_02';
-$VERSION = eval $VERSION if $VERSION =~ /_/;
-
-use vars qw(%input_expr %output_expr $ProtoUsed @InitFileCode $FH $proto_re $Overload $errors $Fallback
-	    $cplusplus $hiertype $WantPrototypes $WantVersionChk $except $WantLineNumbers
-	    $WantOptimize $process_inout $process_argtypes @tm
-	    $dir $filename $filepathname %IncludedFiles
-	    %type_kind %proto_letter
-            %targetable $BLOCK_re $lastline $lastline_no
-            $Package $Prefix @line @BootCode %args_match %defaults %var_types %arg_list @proto_arg
-            $processing_arg_with_types %argtype_seen @outlist %in_out %lengthof
-            $proto_in_this_xsub $scope_in_this_xsub $interface $prepush_done $interface_macro $interface_macro_set
-            $ProtoThisXSUB $ScopeThisXSUB $xsreturn
-            @line_no $ret_type $func_header $orig_args
-	   ); # Add these just to get compilation to happen.
-
 
 sub process_file {
   
@@ -56,7 +38,6 @@ sub process_file {
 	   argtypes => 1,
 	   typemap => [],
 	   output => \*STDOUT,
-	   csuffix => '.c',
 	   %args,
 	  );
 
@@ -73,11 +54,11 @@ sub process_file {
   @XSStack = ({type => 'none'});
   ($XSS_work_idx, $cpp_next_tmp) = (0, "XSubPPtmpAAAA");
   @InitFileCode = ();
-  $FH = Symbol::gensym();
+  $FH = 'File0000' ;
   $proto_re = "[" . quotemeta('\$%&*@;[]') . "]" ;
   $Overload = 0;
   $errors = 0;
-  $Fallback = '&PL_sv_undef';
+  $Fallback = 'PL_sv_undef';
 
   # Most of the 1500 lines below uses these globals.  We'll have to
   # clean this up sometime, probably.  For now, we just pull them out
@@ -96,9 +77,7 @@ sub process_file {
   
   for ($args{filename}) {
     die "Missing required parameter 'filename'" unless $_;
-    $filepathname = $_;
     ($dir, $filename) = (dirname($_), basename($_));
-    $filepathname =~ s/\\/\\\\/g;
     $IncludedFiles{$_}++;
   }
   
@@ -119,16 +98,15 @@ sub process_file {
   my $orig_fh = select();
   
   chdir($dir);
-  my $pwd = cwd();
-  my $csuffix = $args{csuffix};
+  $pwd = cwd();
   
   if ($WantLineNumbers) {
     my $cfile;
     if ( $args{outfile} ) {
       $cfile = $args{outfile};
     } else {
-      $cfile = $args{filename};
-      $cfile =~ s/\.xs$/$csuffix/i or $cfile .= $csuffix;
+      $cfile = $filename;
+      $cfile =~ s/\.xs$/.c/i or $cfile .= ".c";
     }
     tie(*PSEUDO_STDOUT, 'ExtUtils::ParseXS::CountLines', $cfile, $args{output});
     select PSEUDO_STDOUT;
@@ -142,16 +120,16 @@ sub process_file {
 
   push @tm, standard_typemap_locations();
 
-  foreach my $typemap (@tm) {
+  foreach $typemap (@tm) {
     next unless -f $typemap ;
     # skip directories, binary files etc.
     warn("Warning: ignoring non-text typemap file '$typemap'\n"), next
       unless -T $typemap ;
     open(TYPEMAP, $typemap)
       or warn ("Warning: could not open typemap file '$typemap': $!\n"), next;
-    my $mode = 'Typemap';
-    my $junk = "" ;
-    my $current = \$junk;
+    $mode = 'Typemap';
+    $junk = "" ;
+    $current = \$junk;
     while (<TYPEMAP>) {
       next if /^\s*		#/;
         my $line_no = $. + 1;
@@ -194,25 +172,15 @@ sub process_file {
     close(TYPEMAP);
   }
 
-  foreach my $value (values %input_expr) {
-    $value =~ s/;*\s+\z//;
-    # Move C pre-processor instructions to column 1 to be strictly ANSI
-    # conformant. Some pre-processors are fussy about this.
-    $value =~ s/^\s+#/#/mg;
-  }
-  foreach my $value (values %output_expr) {
-    # And again.
-    $value =~ s/^\s+#/#/mg;
+  foreach $key (keys %input_expr) {
+    $input_expr{$key} =~ s/;*\s+\z//;
   }
 
-  my ($cast, $size);
-  our $bal = qr[(?:(?>[^()]+)|\((??{ $bal })\))*]; # ()-balanced
+  $bal = qr[(?:(?>[^()]+)|\((??{ $bal })\))*]; # ()-balanced
   $cast = qr[(?:\(\s*SV\s*\*\s*\)\s*)?]; # Optional (SV*) cast
   $size = qr[,\s* (??{ $bal }) ]x; # Third arg (to setpvn)
 
-  foreach my $key (keys %output_expr) {
-    # We can still bootstrap compile 're', because in code re.pm is 
-    # available to miniperl, and does not attempt to load the XS code.
+  foreach $key (keys %output_expr) {
     use re 'eval';
 
     my ($t, $with_size, $arg, $sarg) =
@@ -226,18 +194,17 @@ sub process_file {
     $targetable{$key} = [$t, $with_size, $arg, $sarg] if $t;
   }
 
-  my $END = "!End!\n\n";		# "impossible" keyword (multiple newline)
+  $END = "!End!\n\n";		# "impossible" keyword (multiple newline)
 
   # Match an XS keyword
   $BLOCK_re= '\s*(' . join('|', qw(
-				   REQUIRE BOOT CASE PREINIT INPUT INIT CODE PPCODE
-				   OUTPUT CLEANUP ALIAS ATTRS PROTOTYPES PROTOTYPE
-				   VERSIONCHECK INCLUDE INCLUDE_COMMAND SCOPE INTERFACE
-				   INTERFACE_MACRO C_ARGS POSTCALL OVERLOAD FALLBACK
+				   REQUIRE BOOT CASE PREINIT INPUT INIT CODE PPCODE OUTPUT
+				   CLEANUP ALIAS ATTRS PROTOTYPES PROTOTYPE VERSIONCHECK INCLUDE
+				   SCOPE INTERFACE INTERFACE_MACRO C_ARGS POSTCALL OVERLOAD FALLBACK
 				  )) . "|$END)\\s*:";
 
   
-  our ($C_group_rex, $C_arg);
+  my ($C_group_rex, $C_arg);
   # Group in C (no support for comments or literals)
   $C_group_rex = qr/ [({\[]
 		       (?: (?> [^()\[\]{}]+ ) | (??{ $C_group_rex }) )*
@@ -264,7 +231,7 @@ sub process_file {
 EOM
 
 
-  print("#line 1 \"$filepathname\"\n")
+  print("#line 1 \"$filename\"\n")
     if $WantLineNumbers;
 
   firstmodule:
@@ -273,21 +240,8 @@ EOM
       my $podstartline = $.;
       do {
 	if (/^=cut\s*$/) {
-	  # We can't just write out a /* */ comment, as our embedded
-	  # POD might itself be in a comment. We can't put a /**/
-	  # comment inside #if 0, as the C standard says that the source
-	  # file is decomposed into preprocessing characters in the stage
-	  # before preprocessing commands are executed.
-	  # I don't want to leave the text as barewords, because the spec
-	  # isn't clear whether macros are expanded before or after
-	  # preprocessing commands are executed, and someone pathological
-	  # may just have defined one of the 3 words as a macro that does
-	  # something strange. Multiline strings are illegal in C, so
-	  # the "" we write must be a string literal. And they aren't
-	  # concatenated until 2 steps later, so we are safe.
-	  #     - Nicholas Clark
-	  print("#if 0\n  \"Skipped embedded POD.\"\n#endif\n");
-	  printf("#line %d \"$filepathname\"\n", $. + 1)
+	  print("/* Skipped embedded POD. */\n");
+	  printf("#line %d \"$filename\"\n", $. + 1)
 	    if $WantLineNumbers;
 	  next firstmodule
 	}
@@ -299,76 +253,21 @@ EOM
       die ("Error: Unterminated pod in $filename, line $podstartline\n")
 	unless $lastline;
     }
-    last if ($Package, $Prefix) =
-      /^MODULE\s*=\s*[\w:]+(?:\s+PACKAGE\s*=\s*([\w:]+))?(?:\s+PREFIX\s*=\s*(\S+))?\s*$/;
+    last if ($Module, $Package, $Prefix) =
+      /^MODULE\s*=\s*([\w:]+)(?:\s+PACKAGE\s*=\s*([\w:]+))?(?:\s+PREFIX\s*=\s*(\S+))?\s*$/;
     
     print $_;
   }
-  unless (defined $_) {
-    warn "Didn't find a 'MODULE ... PACKAGE ... PREFIX' line\n";
-    exit 0; # Not a fatal error for the caller process
-  }
+  die "Didn't find a 'MODULE ... PACKAGE ... PREFIX' line\n" unless defined $_;
 
-  print 'ExtUtils::ParseXS::CountLines'->end_marker, "\n" if $WantLineNumbers;
-
-  print <<"EOF";
+    print <<"EOF";
 #ifndef PERL_UNUSED_VAR
 #  define PERL_UNUSED_VAR(var) if (0) var = var
 #endif
 
 EOF
 
-  print <<"EOF";
-#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
-#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)
-
-/* prototype to pass -Wmissing-prototypes */
-STATIC void
-S_croak_xs_usage(pTHX_ const CV *const cv, const char *const params);
-
-STATIC void
-S_croak_xs_usage(pTHX_ const CV *const cv, const char *const params)
-{
-    const GV *const gv = CvGV(cv);
-
-    PERL_ARGS_ASSERT_CROAK_XS_USAGE;
-
-    if (gv) {
-        const char *const gvname = GvNAME(gv);
-        const HV *const stash = GvSTASH(gv);
-        const char *const hvname = stash ? HvNAME(stash) : NULL;
-
-        if (hvname)
-            Perl_croak(aTHX_ "Usage: %s::%s(%s)", hvname, gvname, params);
-        else
-            Perl_croak(aTHX_ "Usage: %s(%s)", gvname, params);
-    } else {
-        /* Pants. I don't think that it should be possible to get here. */
-        Perl_croak(aTHX_ "Usage: CODE(0x%"UVxf")(%s)", PTR2UV(cv), params);
-    }
-}
-#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE
-
-#ifdef PERL_IMPLICIT_CONTEXT
-#define croak_xs_usage(a,b)	S_croak_xs_usage(aTHX_ a,b)
-#else
-#define croak_xs_usage		S_croak_xs_usage
-#endif
-
-#endif
-
-/* NOTE: the prototype of newXSproto() is different in versions of perls,
- * so we define a portable version of newXSproto()
- */
-#ifdef newXS_flags
-#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
-#else
-#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
-#endif /* !defined(newXS_flags) */
-
-EOF
-
-  print 'ExtUtils::ParseXS::CountLines'->end_marker, "\n" if $WantLineNumbers;
+  print "$ExtUtils::ParseXS::CountLines::SECTION_END_MARKER\n" if $WantLineNumbers;
 
   $lastline    = $_;
   $lastline_no = $.;
@@ -423,21 +322,25 @@ EOF
 	   ." followed by a statement on column one?)")
       if $line[0] =~ /^\s/;
     
-    my ($class, $externC, $static, $ellipsis, $wantRETVAL, $RETVAL_no_return);
-    my (@fake_INPUT_pre);	# For length(s) generated variables
-    my (@fake_INPUT);
-    
     # initialize info arrays
     undef(%args_match);
     undef(%var_types);
     undef(%defaults);
+    undef($class);
+    undef($static);
+    undef($elipsis);
+    undef($wantRETVAL) ;
+    undef($RETVAL_no_return) ;
     undef(%arg_list) ;
     undef(@proto_arg) ;
+    undef(@fake_INPUT_pre) ;	# For length(s) generated variables
+    undef(@fake_INPUT) ;
     undef($processing_arg_with_types) ;
     undef(%argtype_seen) ;
     undef(@outlist) ;
     undef(%in_out) ;
     undef(%lengthof) ;
+    # undef(%islengthof) ;
     undef($proto_in_this_xsub) ;
     undef($scope_in_this_xsub) ;
     undef($interface);
@@ -449,7 +352,7 @@ EOF
     $xsreturn = 0;
 
     $_ = shift(@line);
-    while (my $kwd = check_keyword("REQUIRE|PROTOTYPES|FALLBACK|VERSIONCHECK|INCLUDE(?:_COMMAND)?|SCOPE")) {
+    while ($kwd = check_keyword("REQUIRE|PROTOTYPES|FALLBACK|VERSIONCHECK|INCLUDE")) {
       &{"${kwd}_handler"}() ;
       next PARAGRAPH unless @line ;
       $_ = shift(@line);
@@ -457,7 +360,7 @@ EOF
 
     if (check_keyword("BOOT")) {
       &check_cpp;
-      push (@BootCode, "#line $line_no[@line_no - @line] \"$filepathname\"")
+      push (@BootCode, "#line $line_no[@line_no - @line] \"$filename\"")
 	if $WantLineNumbers && $line[0] !~ /^\s*#\s*line\b/;
       push (@BootCode, @line, "") ;
       next PARAGRAPH ;
@@ -477,8 +380,7 @@ EOF
     blurt ("Error: Function definition too short '$ret_type'"), next PARAGRAPH
       unless @line ;
 
-    $externC = 1 if $ret_type =~ s/^extern "C"\s+//;
-    $static  = 1 if $ret_type =~ s/^static\s+//;
+    $static = 1 if $ret_type =~ s/^static\s+//;
 
     $func_header = shift(@line);
     blurt ("Error: Cannot parse function definition from '$func_header'"), next PARAGRAPH
@@ -494,7 +396,7 @@ EOF
     }
 
     # Check for duplicate function definition
-    for my $tmp (@XSStack) {
+    for $tmp (@XSStack) {
       next unless defined $tmp->{functions}{$Full_func_name};
       Warn("Warning: duplicate function definition '$clean_func_name' detected");
       last;
@@ -504,7 +406,6 @@ EOF
     $DoSetMagic = 1;
 
     $orig_args =~ s/\\\s*/ /g;	# process line continuations
-    my @args;
 
     my %only_C_inlist;		# Not in the signature of Perl function
     if ($process_argtypes and $orig_args =~ /\S/) {
@@ -518,8 +419,8 @@ EOF
 	  my ($pre, $name) = ($arg =~ /(.*?) \s*
 					     \b ( \w+ | length\( \s*\w+\s* \) )
 					     \s* $ /x);
-	  next unless defined($pre) && length($pre);
-	  my $out_type = '';
+	  next unless length $pre;
+	  my $out_type;
 	  my $inout_var;
 	  if ($process_inout and s/^(IN|IN_OUTLIST|OUTLIST|OUT|IN_OUT)\s+//) {
 	    my $type = $1;
@@ -568,14 +469,15 @@ EOF
       my $arg0 = ((defined($static) or $func_name eq 'new')
 		  ? "CLASS" : "THIS");
       unshift(@args, $arg0);
+      ($report_args = "$arg0, $report_args") =~ s/^\w+, $/$arg0/;
     }
     my $extra_args = 0;
     @args_num = ();
     $num_args = 0;
     my $report_args = '';
-    foreach my $i (0 .. $#args) {
+    foreach $i (0 .. $#args) {
       if ($args[$i] =~ s/\.\.\.//) {
-	$ellipsis = 1;
+	$elipsis = 1;
 	if ($args[$i] eq '' && $i == $#args) {
 	  $report_args .= ", ...";
 	  pop(@args);
@@ -619,19 +521,12 @@ EOF
 
     $xsreturn = 1 if $EXPLICIT_RETURN;
 
-    $externC = $externC ? qq[extern "C"] : "";
-
     # print function header
     print Q(<<"EOF");
-#$externC
 #XS(XS_${Full_func_name}); /* prototype to pass -Wmissing-prototypes */
 #XS(XS_${Full_func_name})
 #[[
-##ifdef dVAR
-#    dVAR; dXSARGS;
-##else
 #    dXSARGS;
-##endif
 EOF
     print Q(<<"EOF") if $ALIAS ;
 #    dXSI32;
@@ -639,7 +534,7 @@ EOF
     print Q(<<"EOF") if $INTERFACE ;
 #    dXSFUNCTION($ret_type);
 EOF
-    if ($ellipsis) {
+    if ($elipsis) {
       $cond = ($min_args ? qq(items < $min_args) : 0);
     } elsif ($min_args == $num_args) {
       $cond = qq(items != $min_args);
@@ -652,25 +547,24 @@ EOF
 #    *errbuf = '\0';
 EOF
 
-    if($cond) {
-    print Q(<<"EOF");
+    if ($ALIAS)
+      { print Q(<<"EOF") if $cond }
 #    if ($cond)
-#       croak_xs_usage(cv,  "$report_args");
+#       Perl_croak(aTHX_ "Usage: %s($report_args)", GvNAME(CvGV(cv)));
 EOF
-    } else {
-    # cv likely to be unused
-    print Q(<<"EOF");
-#    PERL_UNUSED_VAR(cv); /* -W */
+    else
+      { print Q(<<"EOF") if $cond }
+#    if ($cond)
+#	Perl_croak(aTHX_ "Usage: $pname($report_args)");
 EOF
-    }
-
-    #gcc -Wall: if an xsub has PPCODE is used
-    #it is possible none of ST, XSRETURN or XSprePUSH macros are used
+    
+    #gcc -Wall: if an xsub has no arguments and PPCODE is used
+    #it is likely none of ST, XSRETURN or XSprePUSH macros are used
     #hence `ax' (setup by dXSARGS) is unused
     #XXX: could breakup the dXSARGS; into dSP;dMARK;dITEMS
     #but such a move could break third-party extensions
-    print Q(<<"EOF") if $PPCODE;
-#    PERL_UNUSED_VAR(ax); /* -Wall */
+    print Q(<<"EOF") if $PPCODE and $num_args == 0;
+#   PERL_UNUSED_VAR(ax); /* -Wall */
 EOF
 
     print Q(<<"EOF") if $PPCODE;
@@ -697,10 +591,10 @@ EOF
       $deferred = "";
       %arg_list = () ;
       $gotRETVAL = 0;
-	
+      
       INPUT_handler() ;
       process_keyword("INPUT|PREINIT|INTERFACE_MACRO|C_ARGS|ALIAS|ATTRS|PROTOTYPE|SCOPE|OVERLOAD") ;
-
+      
       print Q(<<"EOF") if $ScopeThisXSUB;
 #   ENTER;
 #   [[
@@ -859,7 +753,7 @@ EOF
 	next;
       }
       last if $_ eq "$END:";
-      death(/^$BLOCK_re/o ? "Misplaced `$1:'" : "Junk at end of function ($_)");
+      death(/^$BLOCK_re/o ? "Misplaced `$1:'" : "Junk at end of function");
     }
     
     print Q(<<"EOF") if $except;
@@ -882,12 +776,12 @@ EOF
 #
 EOF
 
-    our $newXS = "newXS" ;
-    our $proto = "" ;
+    my $newXS = "newXS" ;
+    my $proto = "" ;
     
     # Build the prototype string for the xsub
     if ($ProtoThisXSUB) {
-      $newXS = "newXSproto_portable";
+      $newXS = "newXSproto";
       
       if ($ProtoThisXSUB eq 2) {
 	# User has specified empty prototype
@@ -899,7 +793,7 @@ EOF
 	  $proto_arg[$min_args] .= ";" ;
 	}
 	push @proto_arg, "$s\@"
-	  if $ellipsis ;
+	  if $elipsis ;
 	
 	$proto = join ("", grep defined, @proto_arg);
       }
@@ -909,20 +803,23 @@ EOF
       }
       $proto = qq{, "$proto"};
     }
-
+    
     if (%XsubAliases) {
       $XsubAliases{$pname} = 0
 	unless defined $XsubAliases{$pname} ;
       while ( ($name, $value) = each %XsubAliases) {
 	push(@InitFileCode, Q(<<"EOF"));
-#        cv = ${newXS}(\"$name\", XS_$Full_func_name, file$proto);
+#        cv = newXS(\"$name\", XS_$Full_func_name, file);
 #        XSANY.any_i32 = $value ;
+EOF
+	push(@InitFileCode, Q(<<"EOF")) if $proto;
+#        sv_setpv((SV*)cv$proto) ;
 EOF
       }
     }
     elsif (@Attributes) {
       push(@InitFileCode, Q(<<"EOF"));
-#        cv = ${newXS}(\"$pname\", XS_$Full_func_name, file$proto);
+#        cv = newXS(\"$pname\", XS_$Full_func_name, file);
 #        apply_attrs_string("$Package", cv, "@Attributes", 0);
 EOF
     }
@@ -930,18 +827,17 @@ EOF
       while ( ($name, $value) = each %Interfaces) {
 	$name = "$Package\::$name" unless $name =~ /::/;
 	push(@InitFileCode, Q(<<"EOF"));
-#        cv = ${newXS}(\"$name\", XS_$Full_func_name, file$proto);
+#        cv = newXS(\"$name\", XS_$Full_func_name, file);
 #        $interface_macro_set(cv,$value) ;
+EOF
+	push(@InitFileCode, Q(<<"EOF")) if $proto;
+#        sv_setpv((SV*)cv$proto) ;
 EOF
       }
     }
-    elsif($newXS eq 'newXS'){ # work around P5NCI's empty newXS macro
-      push(@InitFileCode,
-	   "        ${newXS}(\"$pname\", XS_$Full_func_name, file$proto);\n");
-    }
     else {
       push(@InitFileCode,
-	   "        (void)${newXS}(\"$pname\", XS_$Full_func_name, file$proto);\n");
+	   "        ${newXS}(\"$pname\", XS_$Full_func_name, file$proto);\n");
     }
   }
 
@@ -951,7 +847,6 @@ EOF
 #XS(XS_${Packid}_nil); /* prototype to pass -Wmissing-prototypes */
 #XS(XS_${Packid}_nil)
 #{
-#   dXSARGS;
 #   XSRETURN_EMPTY;
 #}
 #
@@ -960,7 +855,7 @@ EOF
     /* Making a sub named "${Package}::()" allows the package */
     /* to be findable via fetchmethod(), and causes */
     /* overload::Overloaded("${Package}") to return true. */
-    (void)${newXS}("${Package}::()", XS_${Packid}_nil, file$proto);
+    newXS("${Package}::()", XS_${Packid}_nil, file$proto);
 MAKE_FETCHMETHOD_WORK
   }
 
@@ -979,33 +874,17 @@ EOF
 
   print Q(<<"EOF");
 #[[
-##ifdef dVAR
-#    dVAR; dXSARGS;
-##else
 #    dXSARGS;
-##endif
 EOF
 
-  #Under 5.8.x and lower, newXS is declared in proto.h as expecting a non-const
-  #file name argument. If the wrong qualifier is used, it causes breakage with
-  #C++ compilers and warnings with recent gcc.
   #-Wall: if there is no $Full_func_name there are no xsubs in this .xs
   #so `file' is unused
   print Q(<<"EOF") if $Full_func_name;
-##if (PERL_REVISION == 5 && PERL_VERSION < 9)
 #    char* file = __FILE__;
-##else
-#    const char* file = __FILE__;
-##endif
 EOF
 
   print Q("#\n");
 
-  print Q(<<"EOF");
-#    PERL_UNUSED_VAR(cv); /* -W */
-#    PERL_UNUSED_VAR(items); /* -W */
-EOF
-    
   print Q(<<"EOF") if $WantVersionChk ;
 #    XS_VERSION_BOOTCHECK ;
 #
@@ -1043,13 +922,6 @@ EOF
     print "\n    /* End of Initialisation Section */\n\n" ;
   }
 
-  print Q(<<'EOF');
-##if (PERL_REVISION == 5 && PERL_VERSION >= 9)
-#  if (PL_unitcheckav)
-#       call_list(PL_scopestack_ix, PL_unitcheckav);
-##endif
-EOF
-
   print Q(<<"EOF");
 #    XSRETURN_YES;
 #]]
@@ -1062,7 +934,6 @@ EOF
   chdir($orig_cwd);
   select($orig_fh);
   untie *PSEUDO_STDOUT if tied *PSEUDO_STDOUT;
-  close $FH;
 
   return 1;
 }
@@ -1121,12 +992,12 @@ sub print_section {
     # the "do" is required for right semantics
     do { $_ = shift(@line) } while !/\S/ && @line;
 
-    print("#line ", $line_no[@line_no - @line -1], " \"$filepathname\"\n")
+    print("#line ", $line_no[@line_no - @line -1], " \"$filename\"\n")
 	if $WantLineNumbers && !/^\s*#\s*line\b/ && !/^#if XSubPPtmp/;
     for (;  defined($_) && !/^$BLOCK_re/o;  $_ = shift(@line)) {
 	print "$_\n";
     }
-    print 'ExtUtils::ParseXS::CountLines'->end_marker, "\n" if $WantLineNumbers;
+    print "$ExtUtils::ParseXS::CountLines::SECTION_END_MARKER\n" if $WantLineNumbers;
 }
 
 sub merge_section {
@@ -1177,7 +1048,7 @@ sub INPUT_handler {
       print "\tSTRLEN\tSTRLEN_length_of_$2;\n";
       $lengthof{$2} = $name;
       # $islengthof{$name} = $1;
-      $deferred .= "\n\tXSauto_length_of_$2 = STRLEN_length_of_$2;\n";
+      $deferred .= "\n\tXSauto_length_of_$2 = STRLEN_length_of_$2;";
     }
 
     # check for optional initialisation code
@@ -1291,9 +1162,7 @@ sub INTERFACE_handler() {
   TrimWhitespace($in);
 
   foreach (split /[\s,]+/, $in) {
-    my $name = $_;
-    $name =~ s/^$Prefix//;
-    $Interfaces{$name} = $_;
+    $Interfaces{$_} = $_;
   }
   print Q(<<"EOF");
 #	XSFUNCTION = $interface_macro($ret_type,cv,XSANY.any_dptr);
@@ -1369,7 +1238,7 @@ sub OVERLOAD_handler()
       $Overload = 1 unless $Overload;
       my $overload = "$Package\::(".$1 ;
       push(@InitFileCode,
-	   "        (void)${newXS}(\"$overload\", XS_$Full_func_name, file$proto);\n");
+	   "        newXS(\"$overload\", XS_$Full_func_name, file$proto);\n");
     }
   }  
 }
@@ -1381,9 +1250,9 @@ sub FALLBACK_handler()
   
   TrimWhitespace($_) ;
   my %map = (
-	     TRUE => "&PL_sv_yes", 1 => "&PL_sv_yes",
-	     FALSE => "&PL_sv_no", 0 => "&PL_sv_no",
-	     UNDEF => "&PL_sv_undef",
+	     TRUE => "PL_sv_yes", 1 => "PL_sv_yes",
+	     FALSE => "PL_sv_no", 0 => "PL_sv_no",
+	     UNDEF => "PL_sv_undef",
 	    ) ;
   
   # check for valid FALLBACK value
@@ -1463,10 +1332,16 @@ sub SCOPE_handler ()
     death("Error: Only 1 SCOPE declaration allowed per xsub")
       if $scope_in_this_xsub ++ ;
 
-    TrimWhitespace($_);
-    death ("Error: SCOPE: ENABLE/DISABLE")
-        unless /^(ENABLE|DISABLE)\b/i;
-    $ScopeThisXSUB = ( uc($1) eq 'ENABLE' );
+    for (;  !/^$BLOCK_re/o;  $_ = shift(@line)) {
+      next unless /\S/;
+      TrimWhitespace($_) ;
+      if ($_ =~ /^DISABLE/i) {
+	$ScopeThisXSUB = 0
+      } elsif ($_ =~ /^ENABLE/i) {
+	$ScopeThisXSUB = 1
+      }
+    }
+
   }
 
 sub PROTOTYPES_handler ()
@@ -1486,22 +1361,6 @@ sub PROTOTYPES_handler ()
 
   }
 
-sub PushXSStack
-  {
-    # Save the current file context.
-    push(@XSStack, {
-		    type            => 'file',
-		    LastLine        => $lastline,
-		    LastLineNo      => $lastline_no,
-		    Line            => \@line,
-		    LineNo          => \@line_no,
-		    Filename        => $filename,
-		    Filepathname    => $filepathname,
-		    Handle          => $FH,
-		   }) ;
-
-  }
-
 sub INCLUDE_handler ()
   {
     # the rest of the current line should contain a valid filename
@@ -1520,13 +1379,18 @@ sub INCLUDE_handler ()
 
     ++ $IncludedFiles{$_} unless /\|\s*$/ ;
 
-    Warn("The INCLUDE directive with a command is deprecated." .
-         " Use INCLUDE_COMMAND instead!")
-      if /\|\s*$/ ;
+    # Save the current file context.
+    push(@XSStack, {
+		    type		=> 'file',
+		    LastLine        => $lastline,
+		    LastLineNo      => $lastline_no,
+		    Line            => \@line,
+		    LineNo          => \@line_no,
+		    Filename        => $filename,
+		    Handle          => $FH,
+		   }) ;
 
-    PushXSStack();
-
-    $FH = Symbol::gensym();
+    ++ $FH ;
 
     # open the new file
     open ($FH, "$_") or death("Cannot open '$_': $!") ;
@@ -1538,7 +1402,6 @@ sub INCLUDE_handler ()
 EOF
 
     $filename = $_ ;
-    $filepathname = "$dir/$filename";
 
     # Prime the pump by reading the first
     # non-blank line
@@ -1550,51 +1413,7 @@ EOF
 
     $lastline = $_ ;
     $lastline_no = $. ;
-  }
 
-sub INCLUDE_COMMAND_handler ()
-  {
-    # the rest of the current line should contain a valid command
-
-    TrimWhitespace($_) ;
-
-    death("INCLUDE_COMMAND: command missing")
-      unless $_ ;
-
-    death("INCLUDE_COMMAND: pipes are illegal")
-      if /^\s*\|/ or /\|\s*$/ ;
-
-    PushXSStack();
-
-    $FH = Symbol::gensym();
-
-    # If $^X is used in INCLUDE_COMMAND, we know it's supposed to be
-    # the same perl interpreter as we're currently running
-    s/^\s*\$\^X/$^X/;
-
-    # open the new file
-    open ($FH, "-|", "$_")
-      or death("Cannot run command '$_' to include its output: $!") ;
-
-    print Q(<<"EOF");
-#
-#/* INCLUDE_COMMAND:  Including output of '$_' from '$filename' */
-#
-EOF
-
-    $filename = $_ ;
-    $filepathname = "$dir/$filename";
-
-    # Prime the pump by reading the first
-    # non-blank line
-
-    # skip leading blank lines
-    while (<$FH>) {
-      last unless /^\s*$/ ;
-    }
-
-    $lastline = $_ ;
-    $lastline_no = $. ;
   }
 
 sub PopFile()
@@ -1611,11 +1430,7 @@ sub PopFile()
     close $FH ;
 
     $FH         = $data->{Handle} ;
-    # $filename is the leafname, which for some reason isused for diagnostic
-    # messages, whereas $filepathname is the full pathname, and is used for
-    # #line directives.
     $filename   = $data->{Filename} ;
-    $filepathname = $data->{Filepathname} ;
     $lastline   = $data->{LastLine} ;
     $lastline_no = $data->{LastLineNo} ;
     @line       = @{ $data->{Line} } ;
@@ -1964,14 +1779,12 @@ sub map_type {
 
 
 #########################################################
-package
-  ExtUtils::ParseXS::CountLines;
+package ExtUtils::ParseXS::CountLines;
 use strict;
 use vars qw($SECTION_END_MARKER);
 
 sub TIEHANDLE {
   my ($class, $cfile, $fh) = @_;
-  $cfile =~ s/\\/\\\\/g;
   $SECTION_END_MARKER = qq{#line --- "$cfile"};
   
   return bless {buffer => '',
@@ -2009,9 +1822,7 @@ sub UNTIE {
   # This sub does nothing, but is neccessary for references to be released.
 }
 
-sub end_marker {
-  return $SECTION_END_MARKER;
-}
+
 
 
 1;
@@ -2136,32 +1947,9 @@ encountered during processing of the XS file.
 
 =head1 AUTHOR
 
-Based on xsubpp code, written by Larry Wall.
+Original code for ExtUtils::xsubpp script was written by Larry Wall.
 
-Maintained by: 
-
-=over 4
-
-=item *
-
-Ken Williams, <ken@mathforum.org>
-
-=item *
-
-David Golden, <dagolden@cpan.org>
-
-=back
-
-=head1 COPYRIGHT
-
-Copyright 2002-2009 by Ken Williams, David Golden and other contributors.  All
-rights reserved.
-
-This library is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
-
-Based on the ExtUtils::xsubpp code by Larry Wall and the Perl 5
-Porters, which was released under the same license terms.
+Adapted into ExtUtils::ParseXS module by Ken Williams, <ken@mathforum.org>
 
 =head1 SEE ALSO
 
